@@ -8,15 +8,19 @@ from tf_geometric.nn import mean_reducer, max_reducer, sum_reducer
 from tf_geometric.nn.conv.gcn import gcn_mapper, gcn_norm_edge
 
 
-def mean_graph_sage(x, edge_index, edge_weight, neighbor_kernel, self_kernel, bias=None, activation=None,
-                    normalize=False):
+def mean_graph_sage(x, edge_index, edge_weight,
+                    self_kernel,
+                    neighbor_kernel,
+                    bias=None,
+                    activation=None,
+                    concat=True, normalize=False):
     """
 
     :param x: Tensor, shape: [num_nodes, num_features], node features
     :param edge_index: Tensor, shape: [2, num_edges], edge information
     :param edge_weight: Tensor or None, shape: [num_edges]
-    :param neighbor_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
     :param self_kernel: Tensor, shape: [num_features, num_hidden_units], weight
+    :param neighbor_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
     :param bias: Tensor, shape: [num_output_features], bias
     :param activation: Activation function to use.
     :param normalize: If set to :obj:`True`, output features
@@ -39,8 +43,10 @@ def mean_graph_sage(x, edge_index, edge_weight, neighbor_kernel, self_kernel, bi
     neighbor_msg = neighbor_reduced_msg @ neighbor_kernel
     x = x @ self_kernel
 
-    h = tf.concat([neighbor_msg, x], axis=1)
-    # h = x @ self_kernel + neighbor_msg
+    if concat:
+        h = tf.concat([x, neighbor_msg], axis=1)
+    else:
+        h = x + neighbor_msg
 
     if bias is not None:
         h += bias
@@ -75,14 +81,16 @@ def gcn_graph_sage(x, edge_index, edge_weight, kernel, bias=None, activation=Non
     if edge_weight is not None:
         edge_weight = tf.ones([edge_index.shape[1]], dtype=tf.float32)
 
-    updated_edge_index, normed_edge_weight = gcn_norm_edge(edge_index, x.shape[0], edge_weight, cache)
+    num_nodes = tf.shape(x)[0]
+
+    updated_edge_index, normed_edge_weight = gcn_norm_edge(edge_index, num_nodes, edge_weight, cache)
     row, col = updated_edge_index
     repeated_x = tf.gather(x, row)
     neighbor_x = tf.gather(x, col)
 
     neighbor_x = gcn_mapper(repeated_x, neighbor_x, edge_weight=normed_edge_weight)
 
-    reduced_msg = sum_reducer(neighbor_x, row, num_nodes=x.shape[0])
+    reduced_msg = sum_reducer(neighbor_x, row, num_nodes=num_nodes)
 
     h = reduced_msg @ kernel
     if bias is not None:
@@ -97,25 +105,27 @@ def gcn_graph_sage(x, edge_index, edge_weight, kernel, bias=None, activation=Non
     return h
 
 
-def mean_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel, self_kernel,
-                         mlp_bias=None, bias=None, activation=None, normalize=False):
+def mean_pool_graph_sage(x, edge_index, edge_weight,
+                         self_kernel, neighbor_mlp_kernel, neighbor_kernel,
+                         neighbor_mlp_bias=None, bias=None, activation=None,
+                         concat=True, normalize=False):
     """
 
-        :param x: Tensor, shape: [num_nodes, num_features], node features
-        :param edge_index: Tensor, shape: [2, num_edges], edge information
-        :param edge_weight: Tensor or None, shape: [num_edges]
-        :param mlp_kernel: Tensor, shape: [num_features, num_hidden_units]. weight.
-        :param neighbor_kernel: Tensor, shape: [num_hidden_units, num_hidden_units], weight.
-        :param self_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
-        :param mlp_bias: Tensor, shape: [num_hidden_units * 2], bias
-        :param bias: Tensor, shape: [num_output_features], bias.
-        :param activation: Activation function to use.
-        :param normalize: If set to :obj:`True`, output features
-                    will be :math:`\ell_2`-normalized, *i.e.*,
-                    :math:`\frac{\mathbf{x}^{\prime}_i}
-                    {\| \mathbf{x}^{\prime}_i \|_2}`.
-                    (default: :obj:`False`)
-        :return: Updated node features (x), shape: [num_nodes, num_output_features]
+    :param x: Tensor, shape: [num_nodes, num_features], node features
+    :param edge_index: Tensor, shape: [2, num_edges], edge information
+    :param edge_weight: Tensor or None, shape: [num_edges]
+    :param self_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
+    :param neighbor_mlp_kernel: Tensor, shape: [num_features, num_hidden_units]. weight.
+    :param neighbor_kernel: Tensor, shape: [num_hidden_units, num_hidden_units], weight.
+    :param neighbor_mlp_bias: Tensor, shape: [num_hidden_units * 2], bias
+    :param bias: Tensor, shape: [num_output_features], bias.
+    :param activation: Activation function to use.
+    :param normalize: If set to :obj:`True`, output features
+                will be :math:`\ell_2`-normalized, *i.e.*,
+                :math:`\frac{\mathbf{x}^{\prime}_i}
+                {\| \mathbf{x}^{\prime}_i \|_2}`.
+                (default: :obj:`False`)
+    :return: Updated node features (x), shape: [num_nodes, num_output_features]
     """
 
     if edge_weight is not None:
@@ -127,9 +137,9 @@ def mean_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel
 
     neighbor_x = gcn_mapper(repeated_x, neighbor_x, edge_weight=edge_weight)
 
-    h = neighbor_x @ mlp_kernel
-    if mlp_bias is not None:
-        h += mlp_bias
+    h = neighbor_x @ neighbor_mlp_kernel
+    if neighbor_mlp_bias is not None:
+        h += neighbor_mlp_bias
 
     if activation is not None:
         h = activation(h)
@@ -139,7 +149,11 @@ def mean_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel
     from_neighbor = reduced_h @ neighbor_kernel
     from_x = x @ self_kernel
 
-    output = tf.concat([from_neighbor, from_x], axis=1)
+    if concat:
+        output = tf.concat([from_x, from_neighbor], axis=1)
+    else:
+        output = from_x + from_neighbor
+
     if bias is not None:
         output += bias
 
@@ -152,17 +166,19 @@ def mean_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel
     return output
 
 
-def max_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel, self_kernel,
-                        mlp_bias=None, bias=None, activation=None, normalize=False):
+def max_pool_graph_sage(x, edge_index, edge_weight,
+                        self_kernel, neighbor_mlp_kernel, neighbor_kernel,
+                        neighbor_mlp_bias=None, bias=None, activation=None,
+                        concat=True, normalize=False):
     """
 
             :param x: Tensor, shape: [num_nodes, num_features], node features
             :param edge_index: Tensor, shape: [2, num_edges], edge information
             :param edge_weight: Tensor or None, shape: [num_edges]
-            :param mlp_kernel: Tensor, shape: [num_features, num_hidden_units]. weight.
-            :param neighbor_kernel: Tensor, shape: [num_hidden_units, num_hidden_units], weight.
             :param self_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
-            :param mlp_bias: Tensor, shape: [num_hidden_units * 2], bias
+            :param neighbor_mlp_kernel: Tensor, shape: [num_features, num_hidden_units]. weight.
+            :param neighbor_kernel: Tensor, shape: [num_hidden_units, num_hidden_units], weight.
+            :param neighbor_mlp_bias: Tensor, shape: [num_hidden_units * 2], bias
             :param bias: Tensor, shape: [num_output_features], bias.
             :param activation: Activation function to use.
             :param normalize: If set to :obj:`True`, output features
@@ -181,9 +197,9 @@ def max_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel,
 
     neighbor_x = gcn_mapper(repeated_x, neighbor_x, edge_weight=edge_weight)
 
-    h = neighbor_x @ mlp_kernel
-    if mlp_bias is not None:
-        h += mlp_bias
+    h = neighbor_x @ neighbor_mlp_kernel
+    if neighbor_mlp_bias is not None:
+        h += neighbor_mlp_bias
 
     if activation is not None:
         h = activation(h)
@@ -192,7 +208,11 @@ def max_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel,
     from_neighs = reduced_h @ neighbor_kernel
     from_x = x @ self_kernel
 
-    output = tf.concat([from_neighs, from_x], axis=1)
+    if concat:
+        output = tf.concat([from_x, from_neighs], axis=1)
+    else:
+        output = from_x + from_neighs
+
     if bias is not None:
         output += bias
 
@@ -205,25 +225,26 @@ def max_pool_graph_sage(x, edge_index, edge_weight, mlp_kernel, neighbor_kernel,
     return output
 
 
-def lstm_graph_sage(x, edge_index, lstm, neighbor_kernel, self_kernel,
-                    bias=None, activation=None, normalize=False):
+def lstm_graph_sage(x, edge_index, lstm, self_kernel, neighbor_kernel,
+                    bias=None, activation=None,
+                    concat=True, normalize=False):
     """
 
-            :param x: Tensor, shape: [num_nodes, num_features], node features.
-            :param edge_index: Tensor, shape: [2, num_edges], edge information.
-            :param lstm: Long Short-Term Merory.
-            :param neighbor_kernel: Tensor, shape: [num_hidden_units, num_hidden_units], weight.
-            :param self_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
-            :param mlp_bias: Tensor, shape: [num_hidden_units * 2], bias
-            :param bias: Tensor, shape: [num_output_features], bias.
-            :param activation: Activation function to use.
-            :param normalize: If set to :obj:`True`, output features
-                        will be :math:`\ell_2`-normalized, *i.e.*,
-                        :math:`\frac{\mathbf{x}^{\prime}_i}
-                        {\| \mathbf{x}^{\prime}_i \|_2}`.
-                        (default: :obj:`False`)
-            :return: Updated node features (x), shape: [num_nodes, num_output_features]
-            """
+
+    :param x: Tensor, shape: [num_nodes, num_features], node features.
+    :param edge_index: Tensor, shape: [2, num_edges], edge information.
+    :param lstm: Long Short-Term Merory.
+    :param self_kernel: Tensor, shape: [num_features, num_hidden_units], weight.
+    :param neighbor_kernel: Tensor, shape: [num_hidden_units, num_hidden_units], weight.
+    :param bias: Tensor, shape: [num_output_features], bias.
+    :param activation: Activation function to use.
+    :param normalize: If set to :obj:`True`, output features
+                will be :math:`\ell_2`-normalized, *i.e.*,
+                :math:`\frac{\mathbf{x}^{\prime}_i}
+                {\| \mathbf{x}^{\prime}_i \|_2}`.
+                (default: :obj:`False`)
+    :return: Updated node features (x), shape: [num_nodes, num_output_features]
+    """
 
     row, col = edge_index
     row_numpy = row.numpy()
@@ -246,10 +267,14 @@ def lstm_graph_sage(x, edge_index, lstm, neighbor_kernel, self_kernel,
 
     reduced_h = tf.reduce_mean(neighbor_h, axis=1)
 
-    from_neighs = reduced_h @ neighbor_kernel
+    from_neighbor = reduced_h @ neighbor_kernel
     from_x = x @ self_kernel
 
-    output = tf.concat([from_neighs, from_x], axis=1)
+    if concat:
+        output = tf.concat([from_x, from_neighbor], axis=1)
+    else:
+        output = from_x + from_neighbor
+
     if bias is not None:
         output += bias
 
